# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-11 19:53
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import molo.core.blocks
import molo.core.models
import wagtail.wagtailcore.blocks
import wagtail.wagtailcore.fields
import wagtail.wagtailimages.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_squashed_0077_molo_page'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='reactionquestionresponse',
            options={'permissions': (('can_view_response', 'Can view Response'),)},
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='body',
            field=wagtail.wagtailcore.fields.StreamField((('heading', wagtail.wagtailcore.blocks.CharBlock(classname='full title')), ('paragraph', molo.core.blocks.MarkDownBlock()), ('image', wagtail.wagtailimages.blocks.ImageChooserBlock()), ('list', wagtail.wagtailcore.blocks.ListBlock(wagtail.wagtailcore.blocks.CharBlock(label='Item'))), ('numbered_list', wagtail.wagtailcore.blocks.ListBlock(wagtail.wagtailcore.blocks.CharBlock(label='Item'))), ('page', wagtail.wagtailcore.blocks.PageChooserBlock()), ('media', molo.core.models.MoloMediaBlock(icon='media'))), blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[('O', 'Open'), ('C', 'Closed'), ('D', 'Disabled'), ('T', 'Timestamped')], max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='social_media_description',
            field=models.TextField(blank=True, null=True, verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='social_media_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='articlepage',
            name='social_media_title',
            field=models.TextField(blank=True, null=True, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='bannerpage',
            name='external_link',
            field=models.TextField(blank=True, help_text='External link which a banner will link to. eg https://www.google.co.za/', null=True),
        ),
        migrations.AlterField(
            model_name='languagepage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[('O', 'Open'), ('C', 'Closed'), ('D', 'Disabled'), ('T', 'Timestamped')], max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='sectionindexpage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[('O', 'Open'), ('C', 'Closed'), ('D', 'Disabled'), ('T', 'Timestamped')], default='O', max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='commenting_state',
            field=models.CharField(blank=True, choices=[('O', 'Open'), ('C', 'Closed'), ('D', 'Disabled'), ('T', 'Timestamped')], max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='content_rotation_end_date',
            field=models.DateTimeField(blank=True, help_text='The date rotation will end', null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='content_rotation_start_date',
            field=models.DateTimeField(blank=True, help_text='The date rotation will begin', null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='enable_next_section',
            field=models.BooleanField(default=False, help_text="Activate up next section underneath articles in this section will appear with the heading and subheading of that article. The text will say 'next' in order to make the user feel like it's fresh content.", verbose_name='Activate up next section underneath articles'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='enable_recommended_section',
            field=models.BooleanField(default=False, help_text="Underneath the area for 'next articles' recommended articles will appear, with the image + heading + subheading", verbose_name='Activate recommended section underneath articles'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='extra_style_hints',
            field=models.TextField(blank=True, default='', help_text='Styling options that can be applied to this section and all its descendants', null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='friday_rotation',
            field=models.BooleanField(default=False, verbose_name='Friday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='monday_rotation',
            field=models.BooleanField(default=False, verbose_name='Monday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='saturday_rotation',
            field=models.BooleanField(default=False, verbose_name='Saturday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='sunday_rotation',
            field=models.BooleanField(default=False, verbose_name='Sunday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='thursday_rotation',
            field=models.BooleanField(default=False, verbose_name='Thursday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='time',
            field=wagtail.wagtailcore.fields.StreamField((('time', wagtail.wagtailcore.blocks.TimeBlock(required=False)),), blank=True, help_text='The time/s content will be rotated', null=True),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='tuesday_rotation',
            field=models.BooleanField(default=False, verbose_name='Tuesday'),
        ),
        migrations.AlterField(
            model_name='sectionpage',
            name='wednesday_rotation',
            field=models.BooleanField(default=False, verbose_name='Wednesday'),
        ),
        migrations.AlterField(
            model_name='sitelanguage',
            name='locale',
            field=models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese'), ('zu', 'Zulu'), ('xh', 'Xhosa'), ('st', 'Sotho'), ('ve', 'Venda'), ('tn', 'Tswana'), ('ts', 'Tsonga'), ('ss', 'Swati'), ('nr', 'Ndebele')], help_text='Site language', max_length=255, verbose_name='language name'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='content_rotation_end_date',
            field=models.DateTimeField(blank=True, help_text='The date rotation will end', null=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='content_rotation_start_date',
            field=models.DateTimeField(blank=True, help_text='The date rotation will begin', null=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='enable_clickable_tags',
            field=models.BooleanField(default=False, verbose_name='Display tags on Front-end'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='enable_service_directory',
            field=models.BooleanField(default=False, verbose_name='Enable service directory'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='enable_tag_navigation',
            field=models.BooleanField(default=False, help_text='Enable tag navigation. When this is true, the clickable tag functionality will be overriden'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='facebook_sharing',
            field=models.BooleanField(default=False, help_text='Enable this field to allow for sharing to Facebook.', verbose_name='Facebook'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='friday_rotation',
            field=models.BooleanField(default=False, verbose_name='Friday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='monday_rotation',
            field=models.BooleanField(default=False, verbose_name='Monday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='saturday_rotation',
            field=models.BooleanField(default=False, verbose_name='Saturday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='show_only_translated_pages',
            field=models.BooleanField(default=False, help_text='When selecting this option, untranslated pages will not be visible to the front end user when they viewing a child language of the site'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='social_media_links_on_footer_page',
            field=wagtail.wagtailcore.fields.StreamField((('social_media_site', wagtail.wagtailcore.blocks.StructBlock((('title', wagtail.wagtailcore.blocks.CharBlock(required=True)), ('link', wagtail.wagtailcore.blocks.CharBlock(required=True)), ('image', wagtail.wagtailimages.blocks.ImageChooserBlock())))),), blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='sunday_rotation',
            field=models.BooleanField(default=False, verbose_name='Sunday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='thursday_rotation',
            field=models.BooleanField(default=False, verbose_name='Thursday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='time',
            field=wagtail.wagtailcore.fields.StreamField((('time', wagtail.wagtailcore.blocks.TimeBlock(required=False)),), blank=True, help_text='The time/s content will be rotated', null=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='tuesday_rotation',
            field=models.BooleanField(default=False, verbose_name='Tuesday'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='twitter_sharing',
            field=models.BooleanField(default=False, help_text='Enable this field to allow for sharing to Twitter.', verbose_name='Twitter'),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='wednesday_rotation',
            field=models.BooleanField(default=False, verbose_name='Wednesday'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-09-19 09:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0069_add_image_info'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='enable_service_directory',
            field=models.BooleanField(default=False, verbose_name=b'Enable service directory'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='google_places_api_server_key',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='google places server key'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_base_url',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory base url'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_password',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory password'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='service_directory_api_username',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='service directory username'),
        ),
    ]



from .base import *  # noqa


# Disable debug mode

DEBUG = False
ENV = 'prd'
TEMPLATE_DEBUG = False

DEFAULT_SITE_PORT = 80

# Compress static files offline
# http://django-compressor.readthedocs.org/en/latest/settings/#django.conf.settings.COMPRESS_OFFLINE

COMPRESS_OFFLINE = True


# Send notification emails as a background task using Celery,
# to prevent this from blocking web server threads
# (requires the django-celery package):
# http://celery.readthedocs.org/en/latest/configuration.html

# import djcelery
#
# djcelery.setup_loader()
#
# CELERY_SEND_TASK_ERROR_EMAILS = True
# BROKER_URL = 'redis://'


# Use Redis as the cache backend for extra performance
# (requires the django-redis-cache package):
# http://wagtail.readthedocs.org/en/latest/howto/performance.html#cache

# CACHES = {
#     'default': {
#         'BACKEND': 'redis_cache.cache.RedisCache',
#         'LOCATION': '127.0.0.1:6379',
#         'KEY_PREFIX': 'base',
#         'OPTIONS': {
#             'CLIENT_CLASS': 'redis_cache.client.DefaultClient',
#         }
#     }
# }

# Setup for CAS
ENABLE_SSO = True

MIDDLEWARE_CLASSES += [  # noqa: F405
    'molo.core.middleware.MoloCASMiddleware',
    'molo.core.middleware.Custom403Middleware',
]


AUTHENTICATION_BACKENDS = [
    'molo.profiles.backends.MoloProfilesModelBackend',
    'molo.core.backends.MoloModelBackend',
    'django.contrib.auth.backends.ModelBackend',
    'molo.core.backends.MoloCASBackend',
]

CAS_SERVER_URL = ''
CAS_ADMIN_PREFIX = '/admin/'
LOGIN_URL = '/accounts/login/'
CAS_VERSION = '3'

COMPRESS_OFFLINE_CONTEXT = {  # noqa
    'STATIC_URL': STATIC_URL,  # noqa
    'ENV': ENV,  # noqa
}  # noqa

try:
    from .local import *  # noqa
except ImportError:
    pass



import unicodecsv as csv

from django.contrib import admin
from django.http import HttpResponse
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from django.contrib.admin.sites import NotRegistered
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import localtime
from django.conf import settings
from django.db.models import Q

from daterange_filter.filter import DateRangeFilter
from wagtail.contrib.modeladmin.options import ModelAdmin as WagtailModelAdmin
from molo.profiles.admin_views import FrontendUsersAdminView
from molo.profiles.models import (
    UserProfile, SecurityQuestion, SecurityAnswer, SecurityQuestionIndexPage)

from import_export.admin import ImportExportModelAdmin
from import_export.fields import Field
from import_export.widgets import DateTimeWidget, DateWidget
from import_export.resources import ModelResource
from import_export.results import RowResult

from wagtail.wagtailcore.models import Site


try:
    admin.site.unregister(User)
except NotRegistered:
    pass


def download_as_csv(ProfileUserAdmin, request, queryset):
    response = HttpResponse(content_type='text/csv', charset='utf-8')
    response['Content-Disposition'] = 'attachment;filename=export.csv'
    writer = csv.writer(response, encoding='utf-8')
    user_model_fields = UserAdmin.list_display + ('date_joined', )
    profile_fields = ('alias', 'mobile_number')
    field_names = user_model_fields + profile_fields
    writer.writerow(field_names)
    for obj in queryset:
        obj.date_joined = obj.date_joined.strftime("%Y-%m-%d %H:%M")
        writer.writerow(
            [getattr(obj, field) for field in user_model_fields] +
            [getattr(obj.profile, field) for field in profile_fields])
    return response


download_as_csv.short_description = "Download selected as csv"


class UserProfileInlineModelAdmin(admin.StackedInline):
    model = UserProfile


class ProfileUserAdmin(UserAdmin):
    list_display = UserAdmin.list_display + (
        'date_joined', '_alias', '_mobile_number',
        '_date_of_birth', '_gender', '_uuid')

    list_filter = UserAdmin.list_filter + ('date_joined', )

    actions = [download_as_csv]

    def _alias(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.alias:
            return obj.profile.alias
        return ''

    def _uuid(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.uuid:
            return obj.profile.uuid
        return ''

    def _mobile_number(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.mobile_number:
            return obj.profile.mobile_number
        return ''

    def _date_of_birth(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.date_of_birth:
            return obj.profile.date_of_birth
        return ''

    def _gender(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.gender:
            return obj.profile.gender
        return ''

    def _site(self, obj, *args, **kwargs):
        if hasattr(obj, 'profile') and obj.profile.site:
            return obj.profile.site
        return ''


# Below here is for Wagtail Admin
class FrontendUsersDateRangeFilter(DateRangeFilter):
    template = 'admin/frontend_users_date_range_filter.html'


class CustomUsersListFilter(admin.SimpleListFilter):
    title = _('Type')
    parameter_name = 'usertype'

    def lookups(self, request, model_admin):
        return (
            ('frontend', _('Frontend Users')),
            ('admin', _('Admin Users')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'frontend':
            return queryset.filter(is_staff=False, groups__isnull=True)

        if self.value() == 'admin':
            return queryset.exclude(is_staff=False, groups__isnull=True)


class FrontendUsersModelAdmin(WagtailModelAdmin, ProfileUserAdmin):
    model = User
    menu_label = 'Users Export'
    menu_icon = 'user'
    menu_order = 600
    index_view_class = FrontendUsersAdminView
    add_to_settings_menu = True
    list_display = ('username', '_alias', '_mobile_number', '_date_of_birth',
                    'email', 'date_joined', 'is_active', '_site',
                    '_gender', '_uuid')

    list_filter = (
        ('date_joined', FrontendUsersDateRangeFilter), 'is_active',
        CustomUsersListFilter)

    search_fields = ('username', 'profile__uuid',)

    def get_queryset(self, request):
        return User.objects.filter(profile__site=request.site)


class UserProfileModelAdmin(WagtailModelAdmin, ProfileUserAdmin):
    model = UserProfile
    menu_label = 'Site Permissions'
    menu_icon = 'user'
    menu_order = 600
    index_view_class = FrontendUsersAdminView
    add_to_settings_menu = True
    list_display = ('user',)

    search_fields = ('user__username',)

    def get_queryset(self, request):
        return UserProfile.objects.all()


class TzDateTimeWidget(DateTimeWidget):

    def render(self, value, obj):
        if settings.USE_TZ:
            value = localtime(value)
        return super(TzDateTimeWidget, self).render(value, obj)


class MultiSiteUserResource(ModelResource):
    date_of_birth = Field(
        'profile__date_of_birth', 'date_of_birth', widget=DateWidget())
    alias = Field('profile__alias', 'alias')
    mobile_number = Field('profile__mobile_number', 'mobile_number')
    gender = Field('profile__gender', 'gender')
    migrated_username = Field(
        'profile__migrated_username', 'migrated_username')
    security_question_answers = Field(
        'profile__security_question_answers',
        'security_question_answers')
    date_joined = Field(
        'date_joined', 'date_joined', widget=TzDateTimeWidget())
    site = Field('profile__site__pk', 'site')

    class Meta:
        model = User
        exclude = ('id', 'is_superuser', 'groups',
                   'user_permissions', 'is_staff', 'last_login')
        export_order = ('username', 'first_name', 'last_name', 'email',
                        'is_active', 'date_joined', 'mobile_number',
                        'alias', 'date_of_birth', 'gender', 'site',
                        'migrated_username')
        import_id_fields = ['username']
        skip_unchanged = True

    def dehydrate_migrated_username(self, user):
        return user.username

    def dehydrate_security_question_answers(self, user):
        try:
            return [(x.question.title, x.answer)
                    for x in user.profile.securityanswer_set.all()]
        except:  # noqa
            return None

    def export(self, queryset=None, *args, **kwargs):
        qs = self._meta.model.objects.exclude(
            Q(is_staff=True) | Q(is_superuser=True))
        return super(MultiSiteUserResource, self).export(
            qs, *args, **kwargs)

    def get_prefixed_username(self, data):
        return '{}_{}'.format(data['site'], data['username']) \
            if data.get('site') else data['username']

    def before_import_row(self, row, **kwargs):
        row['username'] = self.get_prefixed_username(row)

    def import_row(self, row, instance_loader, *args, **kwargs):
        # Disable updating - we don't want to mistakenly override existing data
        if not User.objects.filter(
                username=self.get_prefixed_username(row)).exists():
            return super(MultiSiteUserResource, self).import_row(
                row, instance_loader, *args, **kwargs)

        row_result = self.get_row_result_class()()
        row_result.import_type = RowResult.IMPORT_TYPE_SKIP
        return row_result

    def import_field(self, field, obj, data):
        if field.attribute == 'profile__security_question_answers':
            site = Site.objects.get(pk=data.get('site'))
            security_index = SecurityQuestionIndexPage.objects.descendant_of(
                site.root_page).first()
            for x in data['security_question_answers']:
                # create the security question if it doesn't already exist
                sq = SecurityQuestion.objects.descendant_of(
                    security_index).filter(title=x[0])
                if not sq.exists():
                    sq = SecurityQuestion(title=x[0])
                    security_index.add_child(instance=sq)
                    sq.save_revision().publish()
                else:
                    sq = sq.first()

                # create the securty answer
                answer = SecurityAnswer(
                    question=sq, answer=x[1], user=obj.profile)
                # save separately so that we can pass in is_import
                answer.save(is_import=True)
        else:
            super(MultiSiteUserResource, self).import_field(field, obj, data)

    def import_obj(self, obj, data, dry_run):
        self.import_field(self.fields['username'], obj, data)
        obj.save()
        if data.get('site'):
            obj.profile.site = Site.objects.get(pk=data.get('site'))
            obj.profile.save()
        super(MultiSiteUserResource, self).import_obj(obj, data, dry_run)

    def after_save_instance(self, instance, using_transactions, dry_run):
        # Save related models
        instance.profile.save()


@admin.register(User)
class ProfilesUserAdmin(ImportExportModelAdmin, ProfileUserAdmin):
    resource_class = MultiSiteUserResource
    inlines = (UserProfileInlineModelAdmin, UserProfileInlineModelAdmin)
    list_display = ProfileUserAdmin.list_display
    actions = ProfileUserAdmin.actions



from __future__ import unicode_literals, absolute_import

import os
import logging

from cookiecutter.config import get_user_config
from cookiecutter.generate import generate_context, generate_files
from cookiecutter.main import expand_abbreviations
from cookiecutter.vcs import clone


def cookiecutter(template, checkout=None, no_input=False, extra_context=None):
    """
    Replacement for cookiecutter's own cookiecutter.

    The difference with cookiecutter's cookiecutter function
    is that this one doesn't automatically str() all the values
    passed along to the template.

    :param template: A directory containing a project template directory,
        or a URL to a git repository.
    :param checkout: The branch, tag or commit ID to checkout after clone.
    :param no_input: Prompt the user at command line for manual configuration?
    :param extra_context: A dictionary of context that overrides default
        and user configuration.
    """

    # Get user config from ~/.cookiecutterrc or equivalent
    # If no config file, sensible defaults from config.DEFAULT_CONFIG are used
    config_dict = get_user_config()

    template = expand_abbreviations(template, config_dict)

    # TODO: find a better way to tell if it's a repo URL
    if 'git@' in template or 'https://' in template:
        repo_dir = clone(
            repo_url=template,
            checkout=checkout,
            clone_to_dir=config_dict['cookiecutters_dir'],
            no_input=no_input
        )
    else:
        # If it's a local repo, no need to clone or copy to your
        # cookiecutters_dir
        repo_dir = template

    context_file = os.path.join(repo_dir, 'cookiecutter.json')
    logging.debug('context_file is {0}'.format(context_file))

    context = generate_context(
        context_file=context_file,
        default_context=config_dict['default_context'],
        extra_context=extra_context,
    )

    # Create project from local context and project template.
    generate_files(
        repo_dir=repo_dir,
        context=context
    )



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth.models import User

from molo.profiles.forms import (
    ForgotPasswordForm, RegistrationForm, ProfilePasswordChangeForm)
from molo.core.tests.base import MoloTestCaseMixin
from molo.profiles.models import SecurityQuestion, SecurityQuestionIndexPage


class RegisterTestCase(MoloTestCaseMixin, TestCase):

    def setUp(self):
        self.mk_main()
        self.user = User.objects.create_user(
            username='tester',
            email='tester@example.com',
            password='tester')

        self.security_index = SecurityQuestionIndexPage(
            title='Security Questions',
            slug='security_questions',
        )
        self.main.add_child(instance=self.security_index)
        self.security_index.save()
        self.question = SecurityQuestion(
            title="How old are you?",
            slug="how-old-are-you",
        )
        self.security_index.add_child(instance=self.question)
        self.question.save()

    def test_register_username_correct(self):
        form_data = {
            'username': 'Jeyabal@-1',
            'password': '1234',
            'terms_and_conditions': True
        }
        form = RegistrationForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertEqual(form.is_valid(), True)

    def test_register_username_incorrect(self):
        form_data = {
            'username': 'Jeyabal#',
            'password': '1234',
            'terms_and_conditions': True

        }
        form = RegistrationForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertEqual(form.is_valid(), False)

    def test_register_password_incorrect(self):
        form_data = {
            'username': 'Jeyabal#',
            'password': '12345',
            'terms_and_conditions': True

        }
        form = RegistrationForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertEqual(form.is_valid(), False)

    def test_password_change_incorrect(self):
        form_data = {
            'old_password': '123',
            'new_password': 'jey123',
            'confirm_password': 'jey123',
        }
        form = ProfilePasswordChangeForm(
            data=form_data,
        )
        self.assertEqual(form.is_valid(), False)

    def test_password_change_correct(self):
        form_data = {
            'old_password': '1234',
            'new_password': '3456',
            'confirm_password': '3456',
        }
        form = ProfilePasswordChangeForm(
            data=form_data,
        )
        self.assertEqual(form.is_valid(), True)

    def test_username_exists(self):
        User.objects.create_user(
            'testing', 'testing@example.com', 'testing')
        form_data = {
            'username': 'testing',
            'password': '1234',
        }
        form = RegistrationForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertFalse(form.is_valid())
        [validation_error] = form.errors.as_data()['username']
        self.assertEqual(
            'Username already exists.', validation_error.message)

    def test_terms_and_conditions_is_required(self):
        form_data = {
            'username': 'test',
            'password': '1234',
        }
        form = RegistrationForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertEqual(form.is_valid(), False)

    def test_security_questions_can_contain_non_ascii(self):
        question_accents = SecurityQuestion(
            title=u'Non-ascii characters? ê ờ ប အ',
            slug='non-ascii-characters',
        )
        self.security_index.add_child(instance=question_accents)
        question_accents.save()
        form_data = {
            'username': 'test',
            'password': '1234',
            'terms_and_conditions': True,
        }
        form = RegistrationForm(
            data=form_data,
            questions=[question_accents]
        )
        self.assertTrue(form.is_valid())
        self.assertEqual(
            form.fields['question_0'].label,
            u'Non-ascii characters? ê ờ ប အ',
        )


class PasswordRecoveryTestCase(MoloTestCaseMixin, TestCase):

    def setUp(self):
        self.mk_main()
        self.user = User.objects.create_user(
            username='tester',
            email='tester@example.com',
            password='tester')

        self.security_index = SecurityQuestionIndexPage(
            title='Security Questions',
            slug='security_questions',
        )
        self.main.add_child(instance=self.security_index)
        self.security_index.save()
        self.question = SecurityQuestion(
            title="How old are you?",
            slug="how-old-are-you",
        )
        self.security_index.add_child(instance=self.question)
        self.question.save()

    def test_username_and_security_answer(self):
        form_data = {
            "username": "tester",
            "question_0": "20"
        }
        form = ForgotPasswordForm(
            data=form_data,
            questions=[self.question, ]
        )
        self.assertEqual(form.is_valid(), True)
